#!/usr/bin/env python3

import boto3
import json
import time
import datetime
import sys
import pickle
import os


def thread_worker(q, current_list_item, results, s_timeout, kwargs, function):

    run_timeout = False

    if s_timeout <= 0:
        run_timout = True

    #q.put("starting process")

    function_results = function(current_list_item, kwargs)

    if function_results == None:
        pass
    else:
        results[current_list_item['Key']] = function_results

    #q.put("finished function")

    #q for status messaging to parent
    #current_list_item: item we want to act on
    #results? Wtf is this for?
    #s_timeout how long to run before dipping out
    #kwargs use if needed



from multiprocessing import Process, Manager, Value, Queue

import os, sys, time, json, datetime, random

from .display_v2 import display
from .queue_management import get_status
from .thread_worker import thread_worker

#========================================================================================================>
#========================================================================================================>
#================================== Multi processor right here ==========================================>
#========================================================================================================>
#========================================================================================================>


def thread_pool_manager(ListToProcess=None, s_timeout=None, m_processes=None, kwargs=None, display_bool=True, function=None):

    #Stuff here
    p_list              = {} # dict that holds process list
    q_list              = {} # dict that holds communications queues
    latest_status       = {}
    more_data           = True
    status              = ''
    cont                = True
    manager             = Manager()
    shared_results      = manager.dict()
    list_size           = len(ListToProcess)
    exception_list      = []
    longest_env_name    = 0
    index_of_longest    = 0
    list_pointer        = 0
    bool_prod           = False
    disp_mod            = 0
    # Manager loop
    while cont or more_data:

        # add more tasks to the list - max processes used here
        while len(p_list) < m_processes:
            #print("attempting to add another process")

            if list_pointer >= list_size:
                #print("no more processes in the list!")
                more_data = False
                break
            else:

                current_list_item = ListToProcess[list_pointer]

                #print(current_list_item)
                q_list[current_list_item['Key']] = Queue()     # Status Queue            # Current item      # shared results    # process timeout   # kwargs
                p = Process(target=thread_worker, args=(q_list[current_list_item['Key']], current_list_item, shared_results, s_timeout, kwargs, function) )
                init_time = time.time()
                p.start()
                p_list[current_list_item['Key']] = (p , init_time)
                list_pointer = list_pointer + 1
                #dict should have a bigger size now

        cont = False

        time.sleep(.05)

        deletion_list = []

        # CHECK IF PROCESSES ARE FINISHED
        for proc in p_list:
            if p_list[proc][0].is_alive():
                cont = True #if any process is still alive set this flag to true
            else:
                try:
                    p_list[proc][0].join()
                    #print("PROC " + str(proc))
                    deletion_list.append(proc)
                    #print(f"Finished Job: {proc}")
                except:
                    print("failed to join this thread. Didn't delete it either.")
                    #input("fuck")

        # remove processes that have been joined. I don't know what problems this could have if I delete my reference to a process that fails to join but isn't aliveself.
        # Problems could arise like having 5 blocked processes. No timeout mechanism either.
        # DELETING PROCESSES THAT HAVE FINISHED. You can't modify the data structure during iteration, at least you can't delete elements from it.
        for p in deletion_list:
            del p_list[p]
            del q_list[p]

        # def get_status(q):
        for q_item in q_list:
            temp = get_status(q_list[q_item])

            if temp == None:
                pass
            else:
                latest_status[q_item] = temp

        disp_mod += 1

        try:
            pass
            if display_bool:
                #only display every 50 iterations. Done because I lowered the delay between each loop 
                if disp_mod % 50 == 0:
                    display(p_list, latest_status, str(list_pointer) + "/" + str(list_size), m_processes, s_timeout, bucket=None)
        except Exception as e:
            print(e)
            pass

    new_dict = {}
    for item in shared_results:

        new_dict[item] = shared_results[item]

    return new_dict
    #-----END WHILE-------------------------------------------------------->



import json, os, time, datetime

max_name_len = 65
#---------------------------------------------------------------------------------------------------->
#------------------------------------------ Display ------------------------------------------------->
#---------------------------------------------------------------------------------------------------->
def sp(num):

    count = 0
    while True:
        print(" ",end = '')
        count = count + 1

        if count >= num:
            break

    pass


# get last status from queue
def get_status(q):

    last = None

    while q.empty() == False:
        last = q.get()

    return last



def display(info, statuses, progress, max_processes=None, timeout=None, bucket=None):
    print(" ")
    print(" ")



    numItems = len(info)

    width = max_processes

    cur = time.time()

    os.system('clear')
    print(" ")
    print("<=== Running Jobs ==================>\n")
    #print("====================================>")
    counter = 0
    for item in info:
        wordlen     = len(str(item))
        worddiff    = max_name_len - wordlen
        #worddiff2   = max_name_len - wordlen

        print(item, end='')
        sp(worddiff) #worddiff

        temp2 = statuses[item]
        #print(f"temp2: {temp2}", end='')
        worddiff2 = 25 - len(temp2)
        try:


            sp(worddiff2)
            #print(len(worddiff2))

            print(temp2, end='')
        except Exception as e:
            print(e, end='')
            pass

        #Calculating duration
        st = info[item][1]
        dur = cur - st

        try:
            #print 5 digits of duration
            #sp(worddiff2) # Print a number of spaces to make an even go at it here
            print('\t', end='')

            print(str(dur)[0:5], end='')
            print("s")
        except:
            print(dur)
    diff = width - numItems
    for i in range(0,diff):
        print("[                           ]")
    print("\n<=========================================>")
    #print("Environment: " + str(bucket))
    print("timeout " + str(timeout))
    print('processes ' + str(max_processes))
    print('Started: ' + progress)
    print(" ")

    time.sleep(.25)



import time, json


def status_update(message, q):
    q.put(message)

    # status_update("getting option settings", q)
    # Basically updates the message


def queue_get_all(q):

    stuff = []

    while q.empty() == False:
        stuff.append(q.get())

    return stuff


# get last status from queue
def get_status(q):

    last = None

    while q.empty() == False:
        last = q.get()

    return last



import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="multi-threader-kypkalorian",
    version="0.0.4",
    author="anonymous_donkey_1337",
    author_email="colin.baxter13@gmail.com",
    description="A package for iterating through s3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kypkalorian/multi-threader",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



